'''
Open source interface to reinforcement learning tasks.
The gym library provides an easy-to-use suite of reinforcement learning tasks.

Agit provides a gym environment wrapper for agent-environment communication in cloud.
'''

import os

if 'CLOUD_PROVIDER' in os.environ and os.environ['CLOUD_PROVIDER'] == 'Agit':
    from agit import gym
    
    with open('./humanoid.xml', 'r', encoding='utf-8') as file:
        xml_string = file.read()
    
    env = gym.make('Humanoid-v3', mujoco_model_xml=xml_string)
else:
    import gym
    
    env = gym.make('Humanoid-v3', xml_file=os.path.abspath('./humanoid.xml'))


observation = env.reset()
current_step = 0

for _ in range(100):
    action = env.action_space.sample()  # your agent here (this takes random actions)
    observation, reward, done, info = env.step(action)
    current_step += 1

    print(f'Current Step: {current_step} -> Reward: {reward}')

    if done:
        observation = env.reset()
        current_step = 0
        print('Reset environment.')

env.close()