import os
import ray

'''
The following switch allows the program to run locally or on the Agit distributed cluster without modifications.
'''
if 'CLOUD_PROVIDER' in os.environ and os.environ['CLOUD_PROVIDER'] == 'Agit':
    from agit import ray_init
    ray_init()
else:
    ray.init()

from ray import tune
from ray.tune.suggest.zoopt import ZOOptSearch
from zoopt import ValueType


def my_objective(config, reporter):
    import time
    time.sleep(0.2)
    for i in range(config["iterations"]):
        reporter(
            timesteps_total=i,
            mean_loss=(config["height"] - 14)**2 - abs(config["width"] - 3))
        time.sleep(0.02)


# Dimension dictionary.
# For continuous dimensions: (continuous, search_range, precision);
# For discrete dimensions: (discrete, search_range, has_order).
dim_dict = {
    "height": (ValueType.CONTINUOUS, [-10, 10], 1e-2),
    "width": (ValueType.DISCRETE, [-10, 10], False)
}

config = {
    "num_samples": 200,  # the number of samples
    "config": {
        "iterations": 10,  # evaluation times
    },
    "stop": {
        "timesteps_total": 10  # custom stop rules
    }
}

# Choose a search algorithm
zoopt_search = ZOOptSearch(
    algo="Asracos",  # only support Asracos algorithm currently
    budget=config["num_samples"],
    dim_dict=dim_dict,
    metric="mean_loss",
    mode="min")

tune.run(my_objective,  # objective
         search_alg=zoopt_search,
         name="zoopt_search",
         **config)
