from ray.tune.examples.mnist_pytorch import get_data_loaders, ConvNet, train, test
from ray import tune
import torch.optim as optim
import os
import ray

'''
The following switch allows the program to run locally or on the Agit distributed cluster without modifications.
'''
if 'CLOUD_PROVIDER' in os.environ and os.environ['CLOUD_PROVIDER'] == 'Agit':
    from agit import ray_init
    ray_init()
else:
    ray.init()

"""
This example runs a small grid search to train a convolutional neural network using PyTorch and Tune.
"""


def train_mnist(config):
    train_loader, test_loader = get_data_loaders()
    model = ConvNet()
    optimizer = optim.SGD(model.parameters(), lr=config["lr"])
    for i in range(10):
        train(model, optimizer, train_loader)
        acc = test(model, test_loader)
        tune.report(mean_accuracy=acc)


analysis = tune.run(
    train_mnist,  # objective
    config={
        "lr": tune.grid_search([0.001, 0.01, 0.1])
    }
)

# Get the best hyperparameters
print("Best config: ", analysis.get_best_config(metric="mean_accuracy"))

# Get a dataframe for analyzing trial results.
df = analysis.dataframe()
